import fs from 'fs';
import path from 'path';
import chalk from 'chalk';
import * as yargs from 'yargs';
import remark from 'remark';
import toc from 'remark-toc';
import jsStringEscape from 'js-string-escape';
import reactPlugin from '@structured-types/react-plugin';
import propTypesPlugin from '@structured-types/prop-types-plugin';
import { hasValue, PropKind, isUnionProp, hasProperties, isStringProp, isClassLikeProp, isArrayProp, isTupleProp, isIndexProp, isFunctionProp, parseFiles } from '@structured-types/api';

const traverseFolder = (filePath, levels = 10, fileName = 'package.json') => {
    const files = fs.readdirSync(filePath);
    if (levels === 0) {
        return null;
    }
    const pckg = files.find((file) => file === fileName);
    if (pckg) {
        return path.resolve(filePath, pckg);
    }
    return traverseFolder(path.resolve(filePath, '..'), levels - 1, fileName);
};
const getRepoPath = (initialPath = './') => {
    const packageFileName = traverseFolder(initialPath);
    if (packageFileName) {
        const packageJSON = JSON.parse(fs.readFileSync(packageFileName, 'utf8'));
        const { repository } = packageJSON;
        if (repository) {
            const url = typeof repository === 'string' ? repository : repository.url;
            const { directory } = repository;
            if (url) {
                const baseName = url.split('.').slice(0, -1).join('.');
                return {
                    repo: `${baseName}${directory ? `/tree/master/${directory}` : ''}`,
                    filePath: packageFileName,
                };
            }
        }
    }
    return {};
};

const trimQuotes = (txt) => txt ? txt.replace(/['"]+/g, '') : txt;
const extractCustomTag = (node, tagName) => {
    const nodes = node.children &&
        node.children.filter((child) => child.type === 'html' &&
            child.value &&
            child.value.startsWith(`<${tagName}`));
    return nodes
        ? nodes
            .map((section) => ({
            section,
            match: section.value
                ? section.value.match(/(\w+)\s*=\s*((["'])(.*?)\3|([^>\s]*)(?=\s|\/>))(?=[^<]*>)/g)
                : undefined,
        }))
            .map(({ section, match }) => ({
            attrs: { section, tagName, node },
            attributes: match
                ? match.map((m) => m.split('=').map((value) => trimQuotes(value)))
                : undefined,
        }))
        : undefined;
};
const inlineNewContent = ({ section, tagName, node }, newContent) => {
    const startTag = `<!-- START-${tagName.toUpperCase()} -->`;
    const endTag = `<!-- END-${tagName.toUpperCase()} -->`;
    const index = node.children ? node.children.indexOf(section) : -1;
    if (index === -1) {
        return;
    }
    let deleteNodes = 0;
    let foundComment = false;
    if (node.children) {
        for (let i = index + 1; i < node.children.length; i += 1) {
            const childNode = node.children[i];
            if (childNode.type == 'html' && childNode.value == startTag) {
                foundComment = true;
            }
            if (foundComment) {
                deleteNodes += 1;
            }
            if (childNode &&
                childNode.type == 'html' &&
                (childNode.value == endTag ||
                    (childNode.value &&
                        childNode.value.toLowerCase().startsWith(`<${tagName}`)))) {
                break;
            }
        }
        node.children.splice(index + 1, deleteNodes, ...[
            {
                type: 'html',
                value: startTag,
            },
            ...newContent,
            {
                type: 'html',
                value: endTag,
            },
        ]);
    }
};

const insertOverview = () => {
    return (node) => {
        const sections = extractCustomTag(node, 'package-section');
        if (sections) {
            sections.forEach(({ attrs, attributes }) => {
                if (attributes) {
                    const file = attributes.find((attribute) => attribute[0] === 'file');
                    const sectionAttr = attributes.find((attribute) => attribute[0] === 'section');
                    const sectionName = sectionAttr
                        ? sectionAttr[1].toLowerCase()
                        : 'overview';
                    if (file) {
                        const content = fs.readFileSync(file[1], 'utf8');
                        const overviewNodes = [];
                        const packageJSONFilename = path.resolve(path.join(path.dirname(file[1]), 'package.json'));
                        if (fs.existsSync(packageJSONFilename)) {
                            const packageJSON = JSON.parse(fs.readFileSync(packageJSONFilename, 'utf8'));
                            const { name, description, repository: { directory }, } = packageJSON;
                            const rootRepoDir = `https://github.com/ccontrols/component-controls/blob/master/${directory}`;
                            overviewNodes.push({
                                type: 'heading',
                                depth: 2,
                                children: [
                                    {
                                        type: 'link',
                                        url: rootRepoDir,
                                        children: [{ type: 'text', value: name }],
                                    },
                                ],
                            });
                            overviewNodes.push({
                                type: 'paragraph',
                                children: [{ type: 'text', value: description }],
                            });
                        }
                        remark()
                            .use(() => (sectionNode) => {
                            let foundDepth = 0;
                            if (sectionNode.children) {
                                for (const section of sectionNode.children) {
                                    if (section.type === 'heading') {
                                        if (section.children &&
                                            section.children.length > 0 &&
                                            section.children[0].type === 'text' &&
                                            section.children[0].value &&
                                            section.children[0].value.toLowerCase() === sectionName) {
                                            foundDepth = section.depth;
                                        }
                                        else if (foundDepth >= section.depth) {
                                            break;
                                        }
                                        else if (foundDepth > 0) {
                                            overviewNodes.push(section);
                                        }
                                    }
                                    else if (foundDepth > 0) {
                                        overviewNodes.push(section);
                                    }
                                }
                            }
                        })
                            .process(content);
                        inlineNewContent(attrs, overviewNodes);
                    }
                }
            });
        }
    };
};

const createPropsRow = ({ name, isOptional, type, value, description }, hasValues) => {
    const children = [
        {
            type: 'tableCell',
            children: name
                ? [
                    {
                        type: 'inlineCode',
                        value: `${name}${isOptional ? '' : '*'}`,
                    },
                ]
                : [],
        },
        {
            type: 'tableCell',
            children: type,
        },
    ];
    if (hasValues) {
        children.push({
            type: 'tableCell',
            children: [
                {
                    type: 'text',
                    value: typeof value !== 'undefined' ? value.toString() : '',
                },
            ],
        });
    }
    children.push({
        type: 'tableCell',
        children: [{ type: 'text', value: description || '' }],
    });
    return {
        type: 'tableRow',
        children,
    };
};
const createPropsTable = (children, title) => {
    const propsTable = [];
    let table = undefined;
    let hasValues = false;
    if (children) {
        if (title) {
            propsTable.push({
                type: 'paragraph',
                children: [
                    {
                        type: 'heading',
                        depth: 3,
                        children: [
                            {
                                type: 'strong',
                                children: [
                                    {
                                        type: 'text',
                                        value: title,
                                    },
                                ],
                            },
                        ],
                    },
                ],
            });
        }
        const columns = [
            { type: 'tableCell', children: [{ type: 'text', value: 'Name' }] },
            { type: 'tableCell', children: [{ type: 'text', value: 'Type' }] },
        ];
        hasValues = children.some((item) => item.value !== undefined);
        if (hasValues) {
            columns.push({
                type: 'tableCell',
                children: [{ type: 'text', value: 'Value' }],
            });
        }
        columns.push({
            type: 'tableCell',
            children: [{ type: 'text', value: 'Description' }],
        });
        table = {
            type: 'table',
            children: [
                {
                    type: 'tableRow',
                    children: columns,
                },
            ],
        };
        propsTable.push({
            type: 'paragraph',
            children: [table],
        });
        // eslint-disable-next-line prefer-spread
        table.children.push(...children.map((child) => createPropsRow(child, hasValues)));
    }
    return { propsTable, table, hasValues };
};

/* eslint-disable prefer-spread */
class ExtractProps {
    constructor(files) {
        this.topLevelProps = {};
        this.repoNames = {};
        this.getPropLink = (key) => {
            const nameParts = key.split('.');
            return this.topLevelProps[nameParts[nameParts.length - 1]];
        };
        this.files = files;
    }
    extractPropTable(props, title) {
        const items = props.map((prop) => ({
            name: prop.name || '',
            isOptional: prop.optional,
            type: this.extractPropType(prop, { extractProperties: true }),
            description: prop.description || '',
            value: hasValue(prop) ? prop.value : undefined,
        }));
        return createPropsTable(items, title);
    }
    extractFunctionDeclaration(prop) {
        const result = [
            {
                type: 'text',
                value: '(',
            },
        ];
        if (prop.parameters) {
            for (let i = 0; i < prop.parameters.length; i += 1) {
                const p = prop.parameters[i];
                if (i > 0) {
                    result.push({
                        type: 'text',
                        value: ', ',
                    });
                }
                if (p.name) {
                    result.push({
                        type: 'inlineCode',
                        value: p.name,
                    });
                    if (!p.optional) {
                        result.push({
                            type: 'text',
                            value: '*',
                        });
                    }
                    result.push({
                        type: 'text',
                        value: ': ',
                    });
                }
                result.push(...this.extractPropType(p));
                if (!p.name && !p.optional) {
                    result.push({
                        type: 'text',
                        value: '*',
                    });
                }
            }
        }
        result.push({
            type: 'text',
            value: ')',
        });
        result.push({
            type: 'text',
            value: ' => ',
        });
        if (prop.returns) {
            result.push(...this.extractPropType(prop.returns));
        }
        else {
            result.push({
                type: 'text',
                value: 'void',
            });
        }
        return result;
    }
    extractFunction(prop, _extractTable = true) {
        if (prop.parameters) {
            const { propsTable, table, hasValues } = this.extractPropTable(prop.parameters, 'parameters');
            if (table && prop.returns && prop.returns.kind !== PropKind.Void) {
                table.children.push(createPropsRow({
                    name: 'returns',
                    isOptional: true,
                    type: this.extractPropType(prop.returns),
                    description: prop.returns.description || '',
                }, hasValues));
            }
            return propsTable;
        }
        return [];
    }
    extractInterface(prop) {
        var _a;
        const result = [];
        if (prop.name) {
            const declaration = {
                type: 'paragraph',
                children: [],
            };
            result.push(declaration);
            if ((_a = prop.extends) === null || _a === void 0 ? void 0 : _a.length) {
                const extendsList = prop.extends.reduce((acc, key, idx) => {
                    const p = this.getPropLink(key);
                    let result;
                    if (p) {
                        result = this.extractPropType(p);
                    }
                    else {
                        result = [
                            {
                                type: 'text',
                                value: key,
                            },
                        ];
                    }
                    if (prop.extends && idx < prop.extends.length - 1) {
                        result.push({
                            type: 'text',
                            value: ', ',
                        });
                    }
                    return [...acc, ...result];
                }, []);
                declaration.children.push({
                    type: 'strong',
                    children: [
                        {
                            type: 'text',
                            value: 'extends ',
                        },
                        ...extendsList,
                    ],
                });
            }
            if (isUnionProp(prop)) {
                result.push(...this.extractPropType(prop));
            }
            else if (hasProperties(prop) && prop.properties) {
                const { propsTable } = this.extractPropTable(prop.properties, 'properties');
                result.push(...propsTable);
            }
        }
        return result;
    }
    propLink(type) {
        var _a;
        const typeText = [
            {
                type: 'text',
                value: type,
            },
        ];
        if (typeof type === 'string') {
            const link = this.getPropLink(type);
            if (link) {
                return [
                    {
                        type: 'link',
                        url: `#${(_a = link.name) === null || _a === void 0 ? void 0 : _a.toLowerCase()}`,
                        children: typeText,
                    },
                ];
            }
        }
        return typeText;
    }
    inlineType(prop) {
        let typeNode = undefined;
        if (typeof prop.type === 'string') {
            typeNode = this.propLink(prop.type);
        }
        else if (prop.kind) {
            typeNode = [
                {
                    type: 'text',
                    value: `${PropKind[prop.kind].toLowerCase()}`,
                },
            ];
        }
        if (prop.name && prop.name !== prop.type) {
            return [
                {
                    type: 'text',
                    value: `${prop.name}${typeNode ? ': ' : ''}`,
                },
                ...(typeNode || []),
            ];
        }
        return typeNode || [];
    }
    typeNode(prop, showValue = true) {
        if (typeof prop.type === 'string') {
            if (typeof prop.parent === 'string') {
                return [
                    ...this.propLink(prop.parent),
                    {
                        type: 'text',
                        value: `.${prop.type}`,
                    },
                ];
            }
            return this.propLink(prop.type);
        }
        if (showValue && hasValue(prop) && prop.value !== undefined) {
            const value = isStringProp(prop)
                ? `"${jsStringEscape(prop.value)}"`
                : prop.value.toString();
            return [
                {
                    type: 'text',
                    value,
                },
            ];
        }
        if (prop.kind) {
            const typeNode = [
                {
                    type: 'text',
                    value: `${PropKind[prop.kind].toLowerCase()}`,
                },
            ];
            if (typeof prop.parent === 'string' && this.getPropLink(prop.parent)) {
                const link = this.propLink(prop.parent);
                if (link.length) {
                    typeNode.push({
                        type: 'text',
                        value: ` (`,
                    });
                    typeNode.push(...link);
                    typeNode.push({
                        type: 'text',
                        value: `)`,
                    });
                }
            }
            return typeNode;
        }
        if (prop.name) {
            return [
                {
                    type: 'text',
                    value: prop.name,
                },
            ];
        }
        return [];
    }
    extractPropType(prop, options) {
        var _a;
        if (prop.parent) {
            const parent = this.getPropLink(prop.parent);
            if (parent && isClassLikeProp(parent)) {
                const p = (_a = parent.properties) === null || _a === void 0 ? void 0 : _a.find((p) => p.name === prop.name);
                if (p) {
                    return this.extractType(p, options);
                }
            }
        }
        return this.extractType(prop, options);
    }
    extractType(prop, options) {
        var _a, _b, _c, _d;
        const { showValue = false, extractProperties = false } = options || {};
        if (isUnionProp(prop) && prop.properties) {
            return [
                {
                    type: 'paragraph',
                    children: (_a = prop.properties) === null || _a === void 0 ? void 0 : _a.reduce((acc, t, idx) => {
                        const r = [...acc, ...this.extractPropType(t, { showValue: true })];
                        if (prop.properties && idx < prop.properties.length - 1) {
                            r.push({ type: 'text', value: ' | ' });
                        }
                        return r;
                    }, []),
                },
            ];
        }
        else if (isClassLikeProp(prop)) {
            const propName = typeof prop.type === 'string' ? prop.type : prop.name;
            const result = [];
            if (typeof propName === 'string' && this.getPropLink(propName)) {
                result.push(...this.propLink(propName));
            }
            else if ((_b = prop.properties) === null || _b === void 0 ? void 0 : _b.length) {
                const typeArguments = prop.properties.reduce((acc, p, idx) => {
                    const result = [...acc, ...this.inlineType(p)];
                    if (prop.properties && idx < prop.properties.length - 1) {
                        result.push({
                            type: 'text',
                            value: ', ',
                        });
                    }
                    return result;
                }, []);
                result.push({
                    type: 'paragraph',
                    children: [
                        {
                            type: 'text',
                            value: '{ ',
                        },
                        ...typeArguments,
                        {
                            type: 'text',
                            value: ' }',
                        },
                    ],
                });
            }
            else if ((_c = prop.generics) === null || _c === void 0 ? void 0 : _c.length) {
                const genericArguments = (_d = prop.generics) === null || _d === void 0 ? void 0 : _d.reduce((acc, p, idx) => {
                    const result = [...acc, ...this.inlineType(p)];
                    if (prop.generics && idx < prop.generics.length - 1) {
                        result.push({
                            type: 'text',
                            value: ', ',
                        });
                    }
                    return result;
                }, []);
                result.push(...this.typeNode(prop));
                if (genericArguments === null || genericArguments === void 0 ? void 0 : genericArguments.length) {
                    result.push(...[
                        {
                            type: 'text',
                            value: '<',
                        },
                        ...genericArguments,
                        {
                            type: 'text',
                            value: '>',
                        },
                    ]);
                }
            }
            else {
                result.push({
                    type: 'text',
                    value: propName,
                });
            }
            return result;
        }
        else if (isArrayProp(prop) && prop.properties) {
            return [
                {
                    type: 'paragraph',
                    children: [
                        ...prop.properties.reduce((acc, p, idx) => {
                            const result = this.extractPropType(p);
                            if (prop.properties && idx < prop.properties.length - 1) {
                                result.push({
                                    type: 'text',
                                    value: ', ',
                                });
                            }
                            return [...acc, ...result];
                        }, []),
                        { type: 'text', value: '[]' },
                    ],
                },
            ];
        }
        else if (isTupleProp(prop) && prop.properties) {
            return [
                {
                    type: 'paragraph',
                    children: [
                        { type: 'text', value: '[' },
                        ...(extractProperties
                            ? prop.properties.reduce((acc, p, idx) => {
                                const result = this.extractPropType(p);
                                if (prop.properties && idx < prop.properties.length - 1) {
                                    result.push({
                                        type: 'text',
                                        value: ', ',
                                    });
                                }
                                return [...acc, ...result];
                            }, [])
                            : []),
                        { type: 'text', value: ']' },
                    ],
                },
            ];
        }
        else if (isIndexProp(prop)) {
            const results = [
                { type: 'text', value: '[' },
                { type: 'paragraph', children: this.extractPropType(prop.index) },
                { type: 'text', value: ']' },
            ];
            if (prop.prop) {
                results.push({ type: 'text', value: ': ' });
                results.push({
                    type: 'paragraph',
                    children: this.extractPropType(prop.prop),
                });
            }
            return results;
            //return this.extractFunctionDeclaration(prop);
        }
        else if (isFunctionProp(prop)) {
            return this.extractFunctionDeclaration(prop);
        }
        return this.typeNode(prop, showValue);
    }
    getSourceLocation(prop) {
        const { filePath } = prop;
        if (filePath) {
            if (!this.repoNames[filePath]) {
                this.repoNames[filePath] = getRepoPath(path.dirname(path.resolve(filePath)));
                if (this.repoNames[filePath].filePath) {
                    this.repoNames[filePath].packageName = JSON.parse(fs.readFileSync(this.repoNames[filePath].filePath || '', 'utf8')).name;
                    this.repoNames[filePath].relativePath = path.relative(path.dirname(this.repoNames[filePath].filePath || './'), filePath);
                }
            }
            if (this.repoNames[filePath]) {
                const { repo, relativePath, packageName } = this.repoNames[filePath];
                if (repo) {
                    const { line } = prop.loc || {};
                    const sourceLocation = filePath.includes('node_modules')
                        ? repo
                        : `${repo}/${relativePath}${line ? `#L${line}` : ''}`;
                    return [
                        {
                            type: 'paragraph',
                            children: [
                                {
                                    type: 'emphasis',
                                    children: [
                                        {
                                            type: 'text',
                                            value: 'defined in ',
                                        },
                                        {
                                            type: 'link',
                                            url: sourceLocation,
                                            children: [
                                                {
                                                    type: 'text',
                                                    value: `${packageName}/${relativePath}`,
                                                },
                                            ],
                                        },
                                    ],
                                },
                            ],
                        },
                    ];
                }
            }
        }
        return [];
    }
    extractPropDefinition(prop) {
        const definition = {
            type: 'paragraph',
            children: [],
        };
        if (prop.kind) {
            definition.children.push({
                type: 'strong',
                children: [
                    {
                        type: 'inlineCode',
                        value: `${prop.extension ? `${prop.extension} ` : ''}${PropKind[prop.kind].toLowerCase()}`,
                    },
                ],
            });
        }
        else if (typeof prop.type === 'string') {
            definition.children.push({
                type: 'strong',
                children: [
                    {
                        type: 'inlineCode',
                        children: this.propLink(prop.type),
                    },
                ],
            });
        }
        const loc = this.getSourceLocation(prop);
        if (loc.length) {
            definition.children.push({
                type: 'text',
                value: ' ',
            });
            definition.children.push(...loc);
        }
        return definition;
    }
    extractTSType(prop) {
        const result = [];
        result.push({
            type: 'heading',
            depth: 2,
            children: [{ type: 'text', value: prop.name }],
        });
        result.push(this.extractPropDefinition(prop));
        if (prop.description) {
            result.push(...prop.description.split('\n').map((d) => ({
                type: 'paragraph',
                children: [
                    {
                        type: 'text',
                        value: d,
                    },
                ],
            })));
        }
        if (isFunctionProp(prop)) {
            result.push(...this.extractFunction(prop));
        }
        else if (hasProperties(prop)) {
            result.push(...this.extractInterface(prop));
        }
        if (prop.examples) {
            const codeExamples = prop.examples.filter((e) => e.content);
            const examples = {
                type: 'paragraph',
                children: [
                    {
                        type: 'heading',
                        depth: 3,
                        children: [
                            {
                                type: 'text',
                                value: `example${codeExamples.length > 1 ? 's' : ''}`,
                            },
                        ],
                    },
                    {
                        type: 'text',
                        value: '\n',
                    },
                ],
            };
            codeExamples.forEach((example) => {
                var _a;
                (_a = examples.children) === null || _a === void 0 ? void 0 : _a.push({
                    type: 'code',
                    value: example.content,
                });
            });
            result.push(examples);
        }
        return result;
    }
    extract(options) {
        var _a;
        const result = [];
        if (this.files) {
            const props = parseFiles(this.files, {
                collectFilePath: true,
                collectHelpers: true,
                collectLinesOfCode: true,
                plugins: [propTypesPlugin, reactPlugin],
                ...options,
            });
            let propKeys = Object.keys(props);
            if ((_a = options.extract) === null || _a === void 0 ? void 0 : _a.length) {
                const names = options.extract;
                propKeys = propKeys.sort((key1, key2) => {
                    return names.indexOf(key1) - names.indexOf(key2);
                });
            }
            propKeys.forEach((key) => {
                if (key !== '__helpers' && key !== '__diagnostics') {
                    this.topLevelProps[key] = props[key];
                }
            });
            const helpers = props.__helpers;
            if (helpers) {
                Object.keys(helpers).forEach((key) => {
                    this.topLevelProps[key] = helpers[key];
                });
            }
            Object.values(this.topLevelProps).forEach((prop) => {
                const nodes = this.extractTSType(prop);
                result.push(...nodes);
            });
        }
        return result;
    }
}

/* eslint-disable prefer-spread */
const insertTSDoc = (fileName) => () => {
    const resolve = (file) => {
        if (file.startsWith('.')) {
            return path.resolve(path.dirname(path.resolve(fileName)), file);
        }
        return require.resolve(file);
    };
    const splitCommaAttribute = (attributes, name) => {
        if (attributes) {
            const attribute = attributes.find((attribute) => attribute[0] === name);
            return attribute
                ? attribute[1]
                    .split(',')
                    .map((f) => f.trim())
                    .filter((f) => f)
                : undefined;
        }
        return undefined;
    };
    return (node) => {
        const sections = extractCustomTag(node, 'api-readme');
        if (sections) {
            sections.forEach(({ attrs, attributes = [] }) => {
                var _a;
                if (attributes) {
                    const newNodes = [];
                    const files = ((_a = splitCommaAttribute(attributes, 'files')) === null || _a === void 0 ? void 0 : _a.map((f) => resolve(f))) ||
                        [];
                    if (!files.length) {
                        const srcFolder = resolve('./src');
                        if (fs.existsSync(srcFolder)) {
                            const sourceFiles = fs.readdirSync(srcFolder);
                            const indexFile = sourceFiles.find((f) => f.startsWith('index'));
                            if (indexFile) {
                                files.push(resolve(path.resolve(srcFolder, indexFile)));
                            }
                        }
                    }
                    if (files.length) {
                        const extractProps = new ExtractProps(files);
                        const options = attributes.reduce((acc, attr) => {
                            let value;
                            const strValue = attr[1];
                            if (strValue === 'true') {
                                value = true;
                            }
                            else if (strValue === 'false') {
                                value = false;
                            }
                            else if (!isNaN(parseFloat(strValue))) {
                                value = parseFloat(strValue);
                            }
                            else if (typeof attr[1] === 'string') {
                                value = splitCommaAttribute(attributes, attr[0]);
                            }
                            return { ...acc, [attr[0]]: value };
                        }, {});
                        const tsNodes = extractProps.extract(options);
                        newNodes.push(...tsNodes);
                    }
                    inlineNewContent(attrs, newNodes);
                }
            });
        }
    };
};

/**
 * create markdown pages from README.md type files
 * command-line arguments
 * @param {string} f - input file name to be processed, default is README.md
 * @param {boolean} t - whether to generate a table of contents, default is true
 * @param {boolean} l - whether to output logs, default is true
 */
var index = () => {
    const args = yargs
        .help()
        .alias('help', 'h')
        .option('f', {
        alias: 'file',
        describe: 'Input file name',
        type: 'string',
        default: 'README.md',
    })
        .option('t', {
        alias: 'toc',
        describe: 'Generate a table of content',
        type: 'boolean',
        default: true,
    })
        .option('l', {
        alias: 'log',
        describe: 'Enabled logs',
        type: 'boolean',
        default: true,
    });
    const options = args.parse(process.argv);
    const fileName = options.f;
    if (options.log) {
        console.log('processing file:', chalk.red(path.resolve(fileName)));
    }
    let r = remark().use(insertTSDoc(fileName)).use(insertOverview);
    if (options.toc) {
        r = r.use(toc, { tight: true });
    }
    r.process(fs.readFileSync(fileName, 'utf8'), function (err, file) {
        if (err) {
            throw err;
        }
        fs.writeFileSync(fileName, String(file));
    });
};

export default index;
